#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""
PloneNewsLetter main class
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string

from Products.CMFCore.CMFCorePermissions import View, ManageProperties
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from Products.CMFCore.utils import getToolByName
from OFS import Folder
from zLOG import LOG, WARNING, INFO, ERROR, DEBUG
from global_symbols import *
from DocumentTemplate.DT_Util import html_quote



# PloneNewsLetter definition for Plone1
factory_type_information = {
  'id'             : 'PloneNewsLetter',
  'portal_type'    : 'PloneNewsLetter',
  'meta_type'      : 'PloneNewsLetter',
  'description'    : 'A PloneNewsLetter type',
  'content_icon'   : 'PloneNewsLetter.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PloneNewsLetter',
  'factory'        : 'addPloneNewsLetter',
  'immediate_view' : 'PloneNewsLetter_edit',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'PloneNewsLetter_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'portal_form/PloneNewsLetter_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'content',
    'name'         : 'Content',
    'action'       : 'portal_form/PloneNewsLetter_contentForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'preview',
    'name'         : 'Preview',
    'action'       : 'portal_form/PloneNewsLetter_previewForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    ),
  }


# PloneNewsLetter definition for Plone2
factory_type_information2 = {
  'id'             : 'PloneNewsLetter',
  'portal_type'    : 'PloneNewsLetter',
  'meta_type'      : 'PloneNewsLetter',
  'description'    : 'A PloneNewsLetter type',
  'content_icon'   : 'PloneNewsLetter.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PloneNewsLetter',
  'factory'        : 'addPloneNewsLetter',
  'immediate_view' : 'PloneNewsLetter_edit2',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/PloneNewsLetter_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'string:${object_url}/PloneNewsLetter_editForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'content',
    'name'         : 'Content',
    'action'       : 'string:${object_url}/PloneNewsLetter_contentForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'preview',
    'name'         : 'Preview',
    'action'       : 'string:${object_url}/PloneNewsLetter_previewForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
    ),
  }
  
# The usual factory
def addPloneNewsLetter(self, id, title = '', REQUEST = {}):
    """
    Factory method for a PloneNewsLetter object
    """
    obj = PloneNewsLetter(id, title)
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')
    

class PloneNewsLetter(PortalContent, DefaultDublinCoreImpl):
    """PloneNewsLetter class"""

    #                                                                           #
    #                            BASIC CLASS BEHAVIOUR                          #
    #                                                                           #

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = factory_type_information['meta_type']                   # Has not to be the same as FTI but it is the most usual case.

    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?
                                                                        # This method makes all class properties & methods protected by View by default


    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        PloneNewsLetter.inheritedAttribute('__init__')(self,)
        self.id = id
        self.title = title
        self.description = ""
        self.newsletter_publication_date=None
        self.newsletter_reference=""
        self.newsletter_email=None
        self.rendered_html=""
        self.rendered_txt=""
        self.content_start_date=None
        self.content_end_date=None
        self.selected_content_types=None
        self.selected_content_paths=None
        self.content_start_date=None
        self.content_end_date=None

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        self.indexObject()              # Initiates the ZCatalog
        

    #                                                                           #
    #                           CONTENT EDITING METHODS                         #
    #                                                                           #



    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(PloneNewsLetter_editPermission, 'edit')
    def edit(self, newsletter_publication_date=None,
                   newsletter_reference="",
                   newsletter_email=""):
        """
        edit(self, newsletter_publication_date=None,
                   newsletter_reference="",
                   newsletter_email="") => object modification method
        """
        # Change attributes
        self.newsletter_publication_date=newsletter_publication_date
        self.newsletter_reference=newsletter_reference
        self.newsletter_email=newsletter_email
        # Reindex
        self.reindexObject()

    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(PloneNewsLetter_editPermission, 'edit_rendered_content')
    def edit_rendered_content(self, rendered_html = '', rendered_txt = ''):
        """
        edit(self, rendered_html = '', rendered_txt = '' ) => object modification method
        """
        # Change attributes
        self.rendered_html = rendered_html
        self.rendered_txt = rendered_txt
        # Reindex
        self.reindexObject()

    #                                                                           #
    #                             MISC METHODS                                  #
    #                                                                           #

    security.declareProtected(PloneNewsLetter_editPermission, 'get_newsletter_email')
    def get_newsletter_email(self):
        """
        get_newsletter_email(self) => gets the newsletter email
        """
        newsletter_email=self.REQUEST.get('newsletter_email',None)
        if newsletter_email:
           pass
        else:
           if self.newsletter_email:
               newsletter_email=self.newsletter_email
           else:
               portal_properties = getToolByName(self, 'portal_properties')
               newsletter_properties = portal_properties.newsletter_properties
               newsletter_email=newsletter_properties.email
        return newsletter_email

    security.declareProtected(PloneNewsLetter_editPermission, 'set_selected_content_types')
    def set_selected_content_types(self, selected_content_types):
        """
        set_selected_content_types(self, content_types) => sets the content types
        """
        if self.selected_content_types!=selected_content_types:
            self.selected_content_types=selected_content_types

    security.declareProtected(PloneNewsLetter_editPermission, 'set_dates')
    def set_dates(self, content_start_date,content_end_date):
        """
        set_dates(self, content_start_date, content_end_date)
        => sets the dates
        """
        if self.content_start_date!=content_start_date:
            self.content_start_date = content_start_date
        if self.content_end_date!=content_end_date:
            self.content_end_date = content_end_date

    security.declareProtected(PloneNewsLetter_editPermission, 'get_selected_content_types')
    def get_selected_content_types(self):
        """
        get_selected_content_types(self)
        => returns the selected content types
        """
        types_tool = getToolByName(self, 'portal_types')
        content_types = types_tool.objectIds()
        portal_properties = getToolByName(self, 'portal_properties')
        newsletter_properties = portal_properties.newsletter_properties
        default_content_types = newsletter_properties.default_content_types
        excluded_content_types = newsletter_properties.excluded_content_types
        selected_content_types=[]
        request_selected_content_types=[]
        if self.REQUEST.has_key('selected_content_types'):
            request_selected_content_types=self.REQUEST.get('selected_content_types')
        if request_selected_content_types:
            # This is the case where the user has submitted the form
            # Any submitted selected values have to be taken into account
            # We take the selections and the newsletter_properties into account
            for type in request_selected_content_types:
                if type in content_types:
                    if type in excluded_content_types:
                        pass
                    else:
                        selected_content_types.append(type)
        elif self.selected_content_types==None:
            # This is the case where no newsletter has been created
            # No selected_content_types have been stored yet
            # We have to look in newsletter_properties for the default settings
            for type in content_types:
               if type in excluded_content_types:
                   pass
               elif type in default_content_types:
                   selected_content_types.append(type)
        else:
            # This is the case where a newsletter has been created
            # The selected_content_types have been stored
            # We take the selections and the newsletter_properties into account
            for type in self.selected_content_types:
                if type in content_types:
                    if type in excluded_content_types:
                        pass
                    else:
                        selected_content_types.append(type)
        return selected_content_types

    security.declareProtected(PloneNewsLetter_editPermission, 'get_selectable_content_types')
    def get_selectable_content_types(self):
        """
        get_content_types(self, content_types) => sets the content types
        """
        types_tool = getToolByName(self, 'portal_types')
        content_types = types_tool.objectIds()
        portal_properties = getToolByName(self, 'portal_properties')
        newsletter_properties = portal_properties.newsletter_properties
        default_content_types = newsletter_properties.default_content_types
        excluded_content_types = newsletter_properties.excluded_content_types
        selectable_content_types=[]
        for type in content_types:
            if type in excluded_content_types:
                pass
            else:
                selectable_content_types.append(type)
        return selectable_content_types

    security.declareProtected(PloneNewsLetter_editPermission, 'create_content_type_list')
    def create_content_type_list(self, result):
        # This method is called with a catalog search result, and returns
        # a list of content sorted by portal type

        # We first put all content in a dict by portal type
        content_dict={}
        for content in result:
           ct=content.portal_type # This is always in the meta data of catalog results
           if content_dict.has_key(ct):
               content_dict[ct].append(content)
           else:
               content_dict[ct]=[content]

        # We want to keep the sort order of the content
        # We want to keep the sort order of the selected_content_types
        # We need the content list by content_type
        content_list=[]
        selected_content_types=self.get_selected_content_types()
        for content_type in selected_content_types:
            if content_dict.has_key(content_type):
                content_list.append({'content_type':content_type,
                                     'content_list':content_dict[content_type]})
        return content_list

    security.declareProtected(PloneNewsLetter_editPermission, 'render_newsletter')
    def render_newsletter(self, selected_content_paths):
        """
        render_newsletter(self, selected_content_paths)

        This method is called to store the paths of the selected content, and
        also to render the txt and html versions of the newsletter.
        """
        self.selected_content_paths=selected_content_paths

        self.rendered_html=self.newsletter_body_html.pt_render()
        self.rendered_txt=self.newsletter_body_txt.pt_render()

    security.declareProtected(PloneNewsLetter_editPermission, 'get_content_type_list')
    def get_content_type_list(self):
        """
        get_content_type_list(self) => 
        
        Once the content has been selected, the paths will be stored in
        the PloneNewsLetter instance. This method returns the actual objects
        that these paths point to.
        """
        documents=[]
        for path in self.selected_content_paths:
            # Code adapted from ZCatalog.py method getObject
            obj = self.aq_parent.unrestrictedTraverse(path)
            if not obj:
                REQUEST=self.REQUEST
                obj = self.resolve_url(path, REQUEST)
            documents.append(obj)
        return self.create_content_type_list(documents)

    security.declareProtected(PloneNewsLetter_editPermission, 'get_content_type_dict')
    def get_content_type_dict(self):
        """
        get_content_type_dict(self) => 
        
        Once the content has been selected, the paths will be stored in
        the PloneNewsLetter instance. This method returns the actual objects
        that these paths point to in a dict.
        """
        content_type_list=self.get_content_type_list()
        content_dict={}
        for info in content_type_list:
            content_dict[info['content_type']]=info['content_list']
        return content_dict

    security.declareProtected(PloneNewsLetter_editPermission, 'search_newsletter_content')
    def search_newsletter_content(self):
        result=[]
        selected_content_types=self.get_selected_content_types()
        if selected_content_types and self.content_start_date and self.content_end_date:
            result = self.custom_search_newsletter(selected_content_types=selected_content_types,
                                                   content_start_date=self.content_start_date,
                                                   content_end_date=self.content_end_date)
        return self.create_content_type_list(result)

    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        ret="%s %s %s" % (self.Title(),
                          self.Description(),'r')
        return ret 

# Class instanciation
InitializeClass(PloneNewsLetter)
